@@ -29,30 +29,27 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: login.c,v 1.67 2005/08/11 11:26:11 kloczek Exp $")
-#include "prototypes.h"
-#include "defines.h"
-#include <sys/stat.h>
-#include <stdio.h>
+#ident "$Id: login.c,v 1.83 2006/03/07 15:47:32 kloczek Exp $"
+
 #include <errno.h>
-#include <pwd.h>
 #include <grp.h>
-#include <signal.h>
 #include <lastlog.h>
-#include "faillog.h"
-#include "failure.h"
-#include "pwauth.h"
-#include "getdef.h"
 #ifdef UT_ADDR
 #include <netdb.h>
 #endif
+#include <pwd.h>
+#include <signal.h>
+#include <stdio.h>
+#include <sys/stat.h>
+#include "defines.h"
+#include "faillog.h"
+#include "failure.h"
+#include "getdef.h"
+#include "prototypes.h"
+#include "pwauth.h"
+#include "exitcodes.h"
 #ifdef USE_PAM
 #include "pam_defs.h"
-static const struct pam_conv conv = {
-	misc_conv,
-	NULL
-};
 
 static pam_handle_t *pamh = NULL;
 
@@ -73,6 +70,9 @@ static pam_handle_t *pamh = NULL;
 #define LASTLOG_FILE "/var/log/lastlog"
 #endif
 
+/*
+ * Global variables
+ */
 const char *hostname = "";
 
 static struct passwd pwent;
@@ -150,7 +150,6 @@ static RETSIGTYPE alarm_handler (int);
  * login -h hostname	(for telnetd, etc.)
  * login -f name	(for pre-authenticated login: datakit, xterm, etc.)
  */
-
 static void usage (void)
 {
 	fprintf (stderr, _("Usage: %s [-p] [name]\n"), Prog);
@@ -172,7 +171,6 @@ static void setup_tty (void)
 	/*
 	 * Add your favorite terminal modes here ...
 	 */
-
 	termio.c_lflag |= ISIG | ICANON | ECHO | ECHOE;
 	termio.c_iflag |= ICRNL;
 
@@ -184,7 +182,6 @@ static void setup_tty (void)
 	 * ttymon invocation prefers this, but these settings won't come into
 	 * effect after the first username login 
 	 */
-
 	STTY (0, &termio);
 }
 
@@ -210,7 +207,6 @@ static void check_nologin (void)
 	 * out for root so she knows to remove the file if she's
 	 * forgotten about it ...
 	 */
-
 	fname = getdef_str ("NOLOGINS_FILE");
 	if (fname != NULL && access (fname, F_OK) == 0) {
 		FILE *nlfp;
@@ -220,7 +216,6 @@ static void check_nologin (void)
 		 * Cat the file if it can be opened, otherwise just
 		 * print a default message
 		 */
-
 		if ((nlfp = fopen (fname, "r"))) {
 			while ((c = getc (nlfp)) != EOF) {
 				if (c == '\n')
@@ -264,7 +259,10 @@ static void check_flags (int argc, char *const *argv)
 
 static void init_env (void)
 {
-	char *cp, *tmp;
+#ifndef USE_PAM
+	char *cp;
+#endif
+	char *tmp;
 
 	if ((tmp = getenv ("LANG"))) {
 		addenv ("LANG", tmp);
@@ -274,26 +272,24 @@ static void init_env (void)
 	 * Add the timezone environmental variable so that time functions
 	 * work correctly.
 	 */
-
 	if ((tmp = getenv ("TZ"))) {
 		addenv ("TZ", tmp);
-	} 
+	}
 #ifndef USE_PAM
-		else if ((cp = getdef_str ("ENV_TZ")))
+	else if ((cp = getdef_str ("ENV_TZ")))
 		addenv (*cp == '/' ? tz (cp) : cp, NULL);
-#endif /* !USE_PAM */
+#endif				/* !USE_PAM */
 	/* 
 	 * Add the clock frequency so that profiling commands work
 	 * correctly.
 	 */
-
 	if ((tmp = getenv ("HZ"))) {
 		addenv ("HZ", tmp);
-	} 
+	}
 #ifndef USE_PAM
-		else if ((cp = getdef_str ("ENV_HZ")))
+	else if ((cp = getdef_str ("ENV_HZ")))
 		addenv (cp, NULL);
-#endif /* !USE_PAM */
+#endif				/* !USE_PAM */
 }
 
 
@@ -321,7 +317,6 @@ static RETSIGTYPE alarm_handler (int sig)
  *	-f - do not perform authentication, user is preauthenticated
  *	-h - the name of the remote host
  */
-
 int main (int argc, char **argv)
 {
 	char username[32];
@@ -340,6 +335,7 @@ int main (int argc, char **argv)
 	int flag;
 	int subroot = 0;
 	int is_console;
+	int err;
 	const char *cp;
 	char *tmp;
 	char fromhost[512];
@@ -434,14 +430,6 @@ int main (int argc, char **argv)
 	if (!isatty (0) || !isatty (1) || !isatty (2))
 		exit (1);	/* must be a terminal */
 
-#if 0
-	/*
-	 * Get the utmp file entry and get the tty name from it. The
-	 * current process ID must match the process ID in the utmp
-	 * file if there are no additional flags on the command line.
-	 */
-	checkutmp (!rflg && !fflg && !hflg);
-#else
 	/*
 	 * Be picky if run by normal users (possible if installed setuid
 	 * root), but not if run by root. This way it still allows logins
@@ -450,7 +438,6 @@ int main (int argc, char **argv)
 	 * entry (will not overwrite remote hostname).  --marekm
 	 */
 	checkutmp (!amroot);
-#endif
 	STRFCPY (tty, utent.ut_line);
 	is_console = console (tty);
 
@@ -508,7 +495,7 @@ int main (int argc, char **argv)
 		setup_tty ();
 
 #ifndef USE_PAM
-		umask (getdef_num ("UMASK", 077));
+		umask (getdef_num ("UMASK", GETDEF_DEFAULT_UMASK));
 
 		{
 			/* 
@@ -517,7 +504,6 @@ int main (int argc, char **argv)
 			 * user may have one for themselves, but otherwise,
 			 * just take what you get.
 			 */
-
 			long limit = getdef_long ("ULIMIT", -1L);
 
 			if (limit != -1)
@@ -529,7 +515,6 @@ int main (int argc, char **argv)
 		 * The entire environment will be preserved if the -p flag
 		 * is used.
 		 */
-
 		if (pflg)
 			while (*envp)	/* add inherited environment, */
 				addenv (*envp++, NULL);	/* some variables change later */
@@ -593,12 +578,13 @@ int main (int argc, char **argv)
 		retcode = pam_start ("login", username, &conv, &pamh);
 		if (retcode != PAM_SUCCESS) {
 			fprintf (stderr,
-				 "login: PAM Failure, aborting: %s\n",
+				 _("login: PAM Failure, aborting: %s\n"),
 				 pam_strerror (pamh, retcode));
 			SYSLOG ((LOG_ERR, "Couldn't initialize PAM: %s",
-				pam_strerror (pamh, retcode)));
+				 pam_strerror (pamh, retcode)));
 			exit (99);
 		}
+
 		/*
 		 * hostname & tty are either set to NULL or their correct values,
 		 * depending on how much we know. We also set PAM's fail delay to
@@ -622,10 +608,10 @@ int main (int argc, char **argv)
 			if (!gethostname (hostn, sizeof (hostn)))
 				snprintf (login_prompt,
 					  sizeof (login_prompt),
-					  "%s login: ", hostn);
+					  _("%s login: "), hostn);
 			else
 				snprintf (login_prompt,
-					  sizeof (login_prompt), "login: ");
+					  sizeof (login_prompt), _("login: "));
 
 			retcode =
 			    pam_set_item (pamh, PAM_USER_PROMPT, login_prompt);
@@ -646,48 +632,94 @@ int main (int argc, char **argv)
 			 * pay attention to failure count and get rid of
 			 * MAX_LOGIN_TRIES?
 			 */
+			failcount = 0;
+			while (1) {
+			  const char *failent_user;
+			  failed = 0;
 
-			retcode = pam_authenticate (pamh, 0);
-			while ((failcount++ < retries) &&
-			       ((retcode == PAM_AUTH_ERR) ||
-				(retcode == PAM_USER_UNKNOWN) ||
-				(retcode == PAM_CRED_INSUFFICIENT) ||
-				(retcode == PAM_AUTHINFO_UNAVAIL))) {
-				pam_get_item (pamh, PAM_USER,
-					      (const void **) &pam_user);
-				SYSLOG ((LOG_NOTICE,
-					"FAILED LOGIN %d FROM %s FOR %s, %s",
-					failcount, hostname, pam_user,
-					pam_strerror (pamh, retcode)));
-#ifdef HAVE_PAM_FAIL_DELAY
-				pam_fail_delay (pamh, 1000000 * delay);
-#endif
-				fprintf (stderr, "Login incorrect\n\n");
-				pam_set_item (pamh, PAM_USER, NULL);
-				retcode = pam_authenticate (pamh, 0);
-			}
-
-			if (retcode != PAM_SUCCESS) {
-				pam_get_item (pamh, PAM_USER,
-					      (const void **) &pam_user);
-
-				if (retcode == PAM_MAXTRIES)
-					SYSLOG ((LOG_NOTICE,
-						"TOO MANY LOGIN TRIES (%d) FROM %s FOR %s, %s",
-						failcount, hostname,
-						pam_user,
-						pam_strerror (pamh, retcode)));
-				else
-					SYSLOG ((LOG_NOTICE,
-						"FAILED LOGIN SESSION FROM %s FOR %s, %s",
-						hostname, pam_user,
-						pam_strerror (pamh, retcode)));
-
-				fprintf (stderr, "\nLogin incorrect\n");
-				pam_end (pamh, retcode);
-				exit (0);
+			  failcount++;
+			  if (delay > 0)
+			    retcode = pam_fail_delay(pamh, 1000000*delay);
+
+			  retcode = pam_authenticate (pamh, 0);
+
+			  pam_get_item (pamh, PAM_USER,
+					(const void **) &pam_user);
+
+			  if (pam_user && pam_user[0]) {
+			    pwd = getpwnam(pam_user);
+			    if (pwd) {
+			      pwent = *pwd;
+			      failent_user = pwent.pw_name;
+			    } else {
+			      if (getdef_bool("LOG_UNKFAIL_ENAB") && pam_user)
+				failent_user = pam_user;
+			      else
+				failent_user = "UNKNOWN";
+			    }
+			  } else {
+			    pwd = NULL;
+			    failent_user = "UNKNOWN";
+			  }
+
+			  if (retcode == PAM_MAXTRIES || failcount >= retries) {
+			    SYSLOG ((LOG_NOTICE,
+				    "TOO MANY LOGIN TRIES (%d)%s FOR `%s'",
+				    failcount, fromhost, failent_user));
+			    fprintf(stderr,
+				    _("Maximum number of tries exceeded (%d)\n"),
+				    failcount);
+			    PAM_END;
+			    exit(0);
+			  } else if (retcode == PAM_ABORT) {
+			    /* Serious problems, quit now */
+			    fprintf(stderr,_("login: abort requested by PAM\n"));
+			    SYSLOG ((LOG_ERR,"PAM_ABORT returned from pam_authenticate()"));
+			    PAM_END;
+			    exit(99);
+			  } else if (retcode != PAM_SUCCESS) {
+			    SYSLOG ((LOG_NOTICE,"FAILED LOGIN (%d)%s FOR `%s', %s",
+				   failcount, fromhost, failent_user,
+				   pam_strerror (pamh, retcode)));
+			    failed = 1;
+			  }
+
+			  if (!failed)
+			    break;
+
+#ifdef WITH_AUDIT
+				{
+					struct passwd *pw;
+					char buf[64];
+
+					audit_fd = audit_open ();
+					pw = getpwnam (username);
+					if (pw) {
+						snprintf (buf, sizeof (buf),
+						  "uid=%d", pw->pw_uid);
+						audit_log_user_message
+						    (audit_fd, AUDIT_USER_LOGIN,
+						     buf, hostname, NULL,
+						     tty, 0);
+					} else {
+						snprintf (buf, sizeof (buf),
+							  "acct=%s", username);
+						audit_log_user_message
+						    (audit_fd, AUDIT_USER_LOGIN,
+						     buf, hostname, NULL,
+						     tty, 0);
+					}
+					close (audit_fd);
+				}
+#endif				/* WITH_AUDIT */
+
+			  fprintf(stderr,"\nLogin incorrect\n");
+
+			  /* Let's give it another go around */
+			  pam_set_item(pamh,PAM_USER,NULL);
 			}
 
+			/* We don't get here unless they were authenticated above */
 			retcode = pam_acct_mgmt (pamh, 0);
 
 			if (retcode == PAM_NEW_AUTHTOK_REQD) {
@@ -803,7 +835,6 @@ int main (int argc, char **argv)
 		 * If you reach this far, your password has been
 		 * authenticated and so on.
 		 */
-
 		if (!failed && pwent.pw_name && pwent.pw_uid == 0
 		    && !is_console) {
 			SYSLOG ((LOG_CRIT, "ILLEGAL ROOT LOGIN %s", fromhost));
@@ -867,7 +898,6 @@ int main (int argc, char **argv)
 		if (--retries <= 0)
 			SYSLOG ((LOG_CRIT, "REPEATED login failures%s",
 				 fromhost));
-#if 1
 		/*
 		 * If this was a passwordless account and we get here, login
 		 * was denied (securetty, faillog, etc.). There was no
@@ -875,16 +905,14 @@ int main (int argc, char **argv)
 		 * guys won't see that the passwordless account exists at
 		 * all).  --marekm
 		 */
-
 		if (pwent.pw_passwd[0] == '\0')
 			pw_auth ("!", username, reason, (char *) 0);
-#endif
+
 		/*
 		 * Wait a while (a la SVR4 /usr/bin/login) before attempting
 		 * to login the user again. If the earlier alarm occurs
 		 * before the sleep() below completes, login will exit.
 		 */
-
 		if (delay > 0)
 			sleep (delay);
 
@@ -936,6 +964,19 @@ int main (int argc, char **argv)
 #endif
 		goto top;	/* go do all this all over again */
 	}
+
+#ifdef WITH_AUDIT
+	{
+		char buf[32];
+
+		audit_fd = audit_open ();
+		snprintf (buf, sizeof (buf), "uid=%d", pwd->pw_uid);
+		audit_log_user_message (audit_fd, AUDIT_USER_LOGIN,
+					buf, hostname, NULL, tty, 1);
+		close (audit_fd);
+	}
+#endif				/* WITH_AUDIT */
+
 #ifndef USE_PAM			/* pam_lastlog handles this */
 	if (getdef_bool ("LASTLOG_ENAB"))	/* give last login and log this one */
 		dolastlog (&lastlog, &pwent, utent.ut_line, hostname);
@@ -969,8 +1010,8 @@ int main (int argc, char **argv)
 	child = fork ();
 	if (child < 0) {
 		/* error in fork() */
-		fprintf (stderr, "login: failure forking: %s",
-			 strerror (errno));
+		fprintf (stderr, _("%s: failure forking: %s"),
+			 Prog, strerror (errno));
 		PAM_END;
 		exit (0);
 	} else if (child) {
@@ -1085,10 +1126,12 @@ int main (int argc, char **argv)
 		SYSLOG ((LOG_INFO, "`%s' logged in %s", username, fromhost));
 #endif
 	closelog ();
-	if ((tmp = getdef_str ("FAKE_SHELL")) != NULL) {
-		shell (tmp, pwent.pw_shell);	/* fake shell */
-	}
-	shell (pwent.pw_shell, (char *) 0);	/* exec the shell finally. */
+	if ((tmp = getdef_str ("FAKE_SHELL")) != NULL)
+		err = shell (tmp, pwent.pw_shell, newenvp); /* fake shell */
+	else
+		/* exec the shell finally */
+		err = shell (pwent.pw_shell, (char *) 0, newenvp);
+	exit (err == ENOENT ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 	/* NOT REACHED */
 	return 0;
 }
