@@ -29,16 +29,20 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: sulogin.c,v 1.20 2005/07/06 11:33:06 kloczek Exp $")
-#include "prototypes.h"
-#include "defines.h"
-#include "getdef.h"
+#ident "$Id: sulogin.c,v 1.25 2006/02/08 10:53:16 kloczek Exp $"
+
+#include <fcntl.h>
+#include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
-#include <pwd.h>
-#include <fcntl.h>
+#include "defines.h"
+#include "getdef.h"
+#include "prototypes.h"
 #include "pwauth.h"
+#include "exitcodes.h"
+/*
+ * Global variables
+ */
 static char name[BUFSIZ];
 static char pass[BUFSIZ];
 
@@ -54,9 +58,9 @@ extern char **environ;
 #endif
 
 /* local function prototypes */
-static RETSIGTYPE catch (int);
+static RETSIGTYPE catch_signals (int);
 
-static RETSIGTYPE catch (int sig)
+static RETSIGTYPE catch_signals (int sig)
 {
 	exit (1);
 }
@@ -73,6 +77,7 @@ static RETSIGTYPE catch (int sig)
 	char *cp;
 	char **envp = environ;
 	TERMIO termio;
+	int err = 0;
 
 #ifdef	USE_TERMIO
 	ioctl (0, TCGETA, &termio);
@@ -150,17 +155,15 @@ static RETSIGTYPE catch (int sig)
 
 	(void) strcpy (name, "root");	/* KLUDGE!!! */
 
-	signal (SIGALRM, catch);	/* exit if the timer expires */
+	signal (SIGALRM, catch_signals);	/* exit if the timer expires */
 	alarm (ALARM);		/* only wait so long ... */
 
 	while (1) {		/* repeatedly get login/password pairs */
 		pw_entry (name, &pwent);	/* get entry from password file */
 		if (pwent.pw_name == (char *) 0) {
-
 			/*
 			 * Fail secure
 			 */
-
 			printf (_("No password entry for 'root'\n"));
 #ifdef	USE_SYSLOG
 			SYSLOG (LOG_WARN, "No password entry for 'root'\n");
@@ -219,6 +222,8 @@ static RETSIGTYPE catch (int sig)
 #ifdef	USE_SYSLOG
 	closelog ();
 #endif
-	shell (pwent.pw_shell, (char *) 0);	/* exec the shell finally. */
+	/* exec the shell finally. */
+	err = shell (pwent.pw_shell, (char *) 0, environ);
+	exit (err == ENOENT ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 	 /*NOTREACHED*/ return (0);
 }
